//
// RGB-Dimmer for a MySensors node - www.mysensors.org to work with VoxCommando as controller - www.voxcommando.com - VoxCommando can control all your Sensors by voice and has also include a Vera Plugin
//
// Changelog 1.1: added fading
//                added program "smooth" (control with child 1: 0=off, 1=on)
//                LED Pins changed to 3,5,6
// Changelog 1.2: replaced the math from all analogWrite(), because we do now use a RGB value (0-255) instead of percent (0-100)
//                adjusted the void smooth() to RGB value
// Changelog 1.3: fade time is now forwarded in the functions
//                new programs "blink" & "sunrise" (control with child 1: 2=blink, 3=sunrise)
//                minor changes in code and bugfixes
// Changelog 1.4: sunrise now only turns ON if actual lightlevel is low 
//                new program "fade to color" (control with child 1: 4=fade2)
// Changelog 1.5: added child 2 to control the speed of fading and programs
//                new program for blinking in one chosen color (control with child 1: 5=blinkcol)
//


#define SN "rgbLED"
#define SV "1.5"

#include <MySensor.h>
#include <SPI.h>

#define RED       3 // pin for red LED 
#define BLUE      5 // pin for blue
#define GREEN     6 // pin for green
#define delayFADE 20
#define delaySMOOTH 100
#define delayBLINK 2000
#define sunFADE 4000 // => 17 minutes to max brightness

static int currentRED = 0;  // Current dim level red
static int currentBLUE = 0;  // Current dim level blue
static int currentGREEN = 0;  // Current dim level green
int smoothSpeed = 50;
int fadeSpeed = 50;
int blinkSpeed = 50;
int preRED;
int preBLUE;
int preGREEN;
int rRED;
int rBLUE;
int rGREEN;
boolean smoothbool = false;
boolean fade2bool1 = false;
boolean fade2bool2 = false;
boolean blinkbool = false;
boolean sendfade = false;
boolean blinkcolbool1 = false;
boolean blinkcolbool2 = false;
unsigned long time2;

MySensor gw;

void setup()
{
	gw.begin(setDimmerStatus, 50); //<- set the second value (Node ID) to your needs. 

	// Send the Sketch Version Information to the Gateway
	gw.sendSketchInfo(SN, SV);
	// Register all sensors to gw (they will be created as child devices)
	gw.present(1, S_DIMMER); // used to call "programs"
        gw.present(2, S_DIMMER);  // used to vary speed of fade and programs
        gw.present(RED, S_DIMMER);
	gw.present(BLUE, S_DIMMER);
        gw.present(GREEN, S_DIMMER);


	pinMode(RED, OUTPUT);
	pinMode(BLUE, OUTPUT);
        pinMode(GREEN, OUTPUT);

	currentRED = gw.loadState(RED);  // read current dim level RED from RAM        
        currentBLUE = gw.loadState(BLUE);  // read current dim level BLUE from RAM
        currentGREEN = gw.loadState(GREEN);  // read current dim level GREEN from RAM
        if (gw.loadState(21) > 0 && gw.loadState(21) <= 100){
          fadeSpeed = gw.loadState(21);
        }
        if (gw.loadState(22) > 0 && gw.loadState(22) <= 100){
          smoothSpeed = gw.loadState(22);
        }
        if (gw.loadState(23) > 0 && gw.loadState(23) <= 100){
          blinkSpeed = gw.loadState(23);
        }
        if (gw.loadState(1) == 0){
          analogWrite(RED, currentRED);          
          analogWrite(BLUE, currentBLUE);
          analogWrite(GREEN, currentGREEN);
        }
        else if (gw.loadState(1) == 1){
          smoothbool = true;
        }
        else if (gw.loadState(1) == 2){
          blinkbool = true;
        }
        else if (gw.loadState(1) == 4){
          fade2bool2 = true;
          preRED = gw.loadState(RED+10);          
          preBLUE = gw.loadState(BLUE+10);
          preGREEN = gw.loadState(GREEN+10);
          analogWrite(RED, currentRED);          
          analogWrite(BLUE, currentBLUE);
          analogWrite(GREEN, currentGREEN);
        }
        else if (gw.loadState(1) == 5){
          fade2bool2 = true;
          currentRED = gw.loadState(RED+10);          
          currentBLUE = gw.loadState(BLUE+10);
          currentGREEN = gw.loadState(GREEN+10);
        }
        rRED = currentRED;        
        rBLUE = currentBLUE;
        rGREEN = currentGREEN;
}

void loop()
{
	gw.process();
          if (smoothbool){
            smooth();
          }
          else if (blinkbool){
            blink();
          }
          else if (fade2bool2){
            fade2(preRED, preBLUE, preGREEN);
          }
          else if (blinkcolbool2){
            blinkcol();
          }
}

void setDimmerStatus(const MyMessage &message)
{
  if (message.type == V_DIMMER){
    if (message.sensor == 1 && message.getByte() == 1){
      smoothbool = true;
      blinkbool = false;
      fade2bool1 = false;
      fade2bool2 = false;
      blinkcolbool1 = false;
      blinkcolbool2 = false;
      sendfade = false;
      gw.saveState(1,1);
    }
    else if (message.sensor == 1 && message.getByte() == 2){
      blinkbool = true;
      smoothbool = false;
      fade2bool1 = false;
      fade2bool2 = false;
      blinkcolbool1 = false;
      blinkcolbool2 = false;
      sendfade = false;
      gw.saveState(1,2);
    }
    else if (message.sensor == 1 && message.getByte() == 3){
      if (gw.loadState(1) == 0 && currentRED + currentBLUE + currentGREEN <= 60){
        fade(255, 200, 50, sunFADE); // here you can set the colour to which will fade sunFADE
      }
    }
    else if (message.sensor == 1 && message.getByte() == 4){
      smoothbool = false;
      blinkbool = false;
      blinkcolbool1 = false;
      blinkcolbool2 = false;
      if (currentRED + currentBLUE + currentGREEN == 0){
        currentRED = gw.loadState(RED+10);        
        currentBLUE = gw.loadState(BLUE+10);
        currentGREEN = gw.loadState(GREEN+10);
        analogWrite(RED, currentRED);        
        analogWrite(BLUE, currentBLUE);
        analogWrite(GREEN, currentGREEN);
      }
      fade2bool1 = true;
      time2 = millis();
      gw.saveState(1,4);
      gw.saveState(RED+10, currentRED);      
      gw.saveState(BLUE+10, currentBLUE);
      gw.saveState(GREEN+10, currentGREEN);
    }
    else if (message.sensor == 1 && message.getByte() == 5){
      blinkbool = false;
      smoothbool = false;
      fade2bool1 = false;
      fade2bool2 = false;
      blinkcolbool1 = true;
      gw.saveState(RED+10, currentRED);
      Serial.println(currentRED);      
      gw.saveState(BLUE+10, currentBLUE);
      Serial.println(currentBLUE);
      gw.saveState(GREEN+10, currentGREEN);
      Serial.println(currentGREEN);
      time2 = millis();
      gw.saveState(1,5);
    }
    else if (message.sensor == 1 && message.getByte() == 0){
      if (gw.loadState(1) != 0){
        gw.saveState(1,0);
      }
      if (blinkcolbool2 || fade2bool2){
        rRED = gw.loadState(RED+10);        
        rBLUE = gw.loadState(BLUE+10);
        rGREEN = gw.loadState(GREEN+10);
      }
      else{
        rRED = gw.loadState(RED);        
        rBLUE = gw.loadState(BLUE);
        rGREEN = gw.loadState(GREEN);
      }
      blinkbool = false;
      smoothbool = false;
      fade2bool1 = false;
      fade2bool2 = false;
      blinkcolbool1 = false; 
      blinkcolbool2 = false;
      sendfade = true;
    } 
    else if (message.sensor == 2){ 
      if (gw.loadState(1) == 0){
        fadeSpeed = message.getByte();  
        gw.saveState(21, fadeSpeed);
      }
      else if (gw.loadState(1) == 1 || gw.loadState(1) == 4){
        smoothSpeed = message.getByte();
        gw.saveState(22, smoothSpeed);
      }
      else if (gw.loadState(1) == 2 || gw.loadState(1) == 5){
        blinkSpeed = message.getByte();
        gw.saveState(23, blinkSpeed);
      }
    }
    else{
      uint8_t incomingDimmerStatus = message.getByte();
      // Change Dimmer level

      if (message.sensor == RED){rRED = incomingDimmerStatus;}      
      else if (message.sensor == BLUE){rBLUE = incomingDimmerStatus;}
      else if (message.sensor == GREEN){rGREEN = incomingDimmerStatus;}
      gw.saveState(message.sensor, message.getByte()); //Save value to RAM
      unsigned long time1 = millis();
      while (millis() < time1 + 200){
        sendfade = false;
        if (gw.process()){
          sendfade = false;
          time1 = time1-200;
        }
        else{
          sendfade = true;
        }                  
      }
    }
    if (sendfade){
      sendfade = false;
      smoothbool = false;
      blinkbool = false;
      if (millis() > time2 + 1000){
        fade2bool1 = false;
        fade2bool2 = false;
        blinkcolbool1 = false;
        blinkcolbool2 = false;
        gw.saveState(1,0);
      }
      if (fade2bool2){
        fade(rRED, rBLUE, rGREEN, delaySMOOTH*smoothSpeed/100);
      }
      else if (blinkcolbool2 == false){
        fade(rRED, rBLUE, rGREEN, delayFADE*fadeSpeed/100);
      }
    }
    if (fade2bool1){
      fade2bool1 = false;
      fade2bool2 = true;
      preRED = rRED;      
      preBLUE = rBLUE;
      preGREEN = rGREEN;
      gw.saveState(RED, rRED);      
      gw.saveState(BLUE, rBLUE);
      gw.saveState(GREEN, rGREEN);
    }
    if (blinkcolbool1){
      blinkcolbool1 = false;
      blinkcolbool2 = true;
    }
  }
}


void fade (int reqRED, int reqGREEN, int reqBLUE, int wait)
{
  int d_red = reqRED - currentRED;  
  int d_blue = reqBLUE - currentBLUE;
  int d_green = reqGREEN - currentGREEN; 
  
  for (int i=0; i <= (max(abs(d_red), max(abs(d_blue), abs(d_green)))); i++){
    unsigned long timePrev = millis();
    while (millis() < timePrev + wait){
      if (gw.process()){
        reqRED = currentRED;        
        reqBLUE = currentBLUE;
        reqGREEN = currentGREEN;
        i = max(abs(d_red), max(abs(d_blue), abs(d_green)));
      }
    }
    if (reqRED != currentRED){
      currentRED = currentRED + d_red/abs(d_red);
      analogWrite(RED, currentRED);  
    }  
    if (reqBLUE != currentBLUE){
      currentBLUE = currentBLUE + d_blue/abs(d_blue); 
      analogWrite(BLUE, currentBLUE);  
    }
    if (reqGREEN != currentGREEN){
      currentGREEN = currentGREEN + d_green/abs(d_green);
      analogWrite(GREEN, currentGREEN);    
      
    }
  } 
}
void smooth(){ 
  if (smoothbool){fade(255,0,0, delaySMOOTH*smoothSpeed/100);}
  if (smoothbool){fade(128,128,0, delaySMOOTH*smoothSpeed/100);}
  if (smoothbool){fade(0,255,0, delaySMOOTH*smoothSpeed/100);}
  if (smoothbool){fade(0,128,128, delaySMOOTH*smoothSpeed/100);}
  if (smoothbool){fade(0,0,255, delaySMOOTH*smoothSpeed/100);}
  if (smoothbool){fade(128,0,128, delaySMOOTH*smoothSpeed/100);}
}

void blink(){
  if (blinkbool){
    fade(255,0,0,0);
    pause();
  }
  if (blinkbool){
    fade(128,128,0,0);
    pause();
  }
  if (blinkbool){
    fade(0,255,0,0);
    pause();
  }
  if (blinkbool){
    fade(0,128,128,0);
    pause();
  }
  if (blinkbool){
    fade(0,0,255,0);
    pause();
  }
  if (blinkbool){
    fade(128,0,128,0);
    pause();
  }
}
void pause(){
  unsigned long timePrev2 = millis();
  while (millis() < timePrev2 + delayBLINK*blinkSpeed/100){
    if (gw.process()){}
  }
}

void fade2(int reqRED, int reqBLUE, int reqGREEN){
  preRED = currentRED;  
  preBLUE = currentBLUE;
  preGREEN = currentGREEN;
  fade (reqRED, reqBLUE, reqGREEN, delaySMOOTH*smoothSpeed/100);
}

void blinkcol(){
  fade(0,0,0,0);
  pause();
  fade(rRED, rBLUE, rGREEN,0);
  pause();
}
